SparseFormat

Enum SparseFormat 

#[non_exhaustive]
pub enum SparseFormat { Coo, Csr, Csc, }
Sparse matrix storage format

Variants (Non-exhaustive)

This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.


Coo

Coordinate format (COO)

Stores explicit (row, col, value) triplets. Best for: construction, format conversion, random access insertion Storage: O(3 * nnz)



Csr

Compressed Sparse Row (CSR)

Row pointers + column indices + values. Best for: row slicing, SpMV, most sparse operations Storage: O(2 * nnz + nrows + 1)



Csc

Compressed Sparse Column (CSC)

Column pointers + row indices + values. Best for: column slicing, transposed operations Storage: O(2 * nnz + ncols + 1)

Implementations

Source

impl SparseFormat

pub fn is_row_major(&self) -> bool

Returns true if format is efficient for row operations

pub fn is_col_major(&self) -> bool

Returns true if format is efficient for column operations

pub fn name(&self) -> &'static str

Returns the format name as a string